#include "CHierScatter.H"
#include "CFlatScatter.H"
#include "ogl.H"
#include "cluster.H"
#include "XmdvTool.H"
#include "colormap.H"
#include "magnify.H"
#include "brush_oper.H"



CHierScatter::CHierScatter(CHierDisplayInformation* hier_info) : CHierDisplay(hier_info)
{
}


//draw the mean and band of one cluster
void CHierScatter::DrawSingleCluster(Cluster &c, bool brushed)
{
	//draw band
    if ( hier_info->SHOW_BANDS )
		DrawSingleNodeBand(c, hier_info->density, brushed);

    //draw mean
    if ( hier_info->SHOW_MEANS )
    {
        if ( brushed )
            canvas->SetForeground(interface_color[SBB], current_opacity);
	    if ( ! brushed )
	    {
            int color_idx = int(c.color*(CMAPSIZE-1));
            canvas->SetForeground( hier_info->CMAP[color_idx], current_opacity);
        }

	    canvas->BeginDraw();
	    glBegin(GL_POINTS);

        for (int j=0; j<dataset->dims; j++) 
	    {
            for (int k=0; k<dataset->dims; k++)
		    {
/*for fisheye view
			    Vec2 p = canvas->FisheyeMapping(CFlatScatter::XMAP(j,c.mean[j]), CFlatScatter::YMAP(k,c.mean[k]));
				glVertex2f(p.X, p.Y);
*/
				glVertex2f((float) XMAP(j,c.mean[j]), (float) YMAP(k,c.mean[k]) );
		    }
        }

	    glEnd();
	    canvas->EndDraw();
    }
    
}

/*
//draw the mean and band of one cluster
void CHierScatter::DrawSingleCluster(Cluster &c, bool brushed)
{
	//draw band
    if ( hier_info->SHOW_BANDS )
		DrawSingleNodeBand(c, hier_info->density, brushed);

    //draw mean
    if ( hier_info->SHOW_MEANS )
    {
        if ( brushed )
            canvas->SetForeground(interface_color[SBB], current_opacity);
	    if ( ! brushed )
	    {
            int color_idx = int(c.color*(CMAPSIZE-1));
            canvas->SetForeground( hier_info->CMAP[color_idx], current_opacity);
        }

	    canvas->BeginDraw();
	    glBegin(GL_POINTS);

        for (int j=0; j<dataset->dims; j++) 
	    {
            for (int k=0; k<dataset->dims; k++)
		    {
			    glVertex2f( CFlatScatter::XMAP(j,c.mean[j]), CFlatScatter::YMAP(k,c.mean[k]) );
		    }
        }

	    glEnd();
	    canvas->EndDraw();
    }
    
}
*/
//draw a band for one cluster
void CHierScatter::DrawSingleNodeBand(Cluster &c, double density, bool is_brushed)
{
    //if the cluster has only one children, there is no band.
    if ( c.num_children == 0 ) 
        return;
	
    if (hier_info->COLORED_BANDS) 
    {
        int color_idx = (int)(c.color*(CMAPSIZE-1));
        canvas->SetForeground( hier_info->CMAP[color_idx], current_opacity);
    }
    else
        canvas->SetForeground(interface_color[BRUSH1+2]);

	//draw band
    for (int j=0; j<dataset->dims; j++) 
	{
        for (int k=0; k<dataset->dims; k++)
		{
			p[0].X = XMAP(j,c.mean[j]);
			p[0].Y = YMAP(k,c.mean[k]);
			p[1].X = XMAP(j,c.min[j]);
			p[1].Y = YMAP(k,c.min[k]);
			p[2].X = XMAP(j,c.min[j]);
			p[2].Y = YMAP(k,c.max[k]);
			p[3].X = XMAP(j,c.max[j]);
			p[3].Y = YMAP(k,c.max[k]);
			p[4].X = XMAP(j,c.max[j]);
			p[4].Y = YMAP(k,c.min[k]);
			if ( fabs(1 - hier_info->extent_scaling_fac ) > 1e-4 )
			{
				for ( int i = 1; i <= 4; i++ )
				{
					p[i].Y = (1-hier_info->extent_scaling_fac)*p[0].Y + (hier_info->extent_scaling_fac*p[i].Y);
					p[i].X = (1-hier_info->extent_scaling_fac)*p[0].X + (hier_info->extent_scaling_fac*p[i].X);
				}
            }

			canvas->DrawTransparentSquare2(p, 0.0, density, 0);
		}
    }
}


void CHierScatter::SetParameters()
{
    CHierDisplay::SetParameters();
	p.SetLen(5);
}


void CHierScatter::FreeSpace()
{
    glPointSize(1.0);
}


/*
// draw the grids and labels of the plots
void DrawScatterGridAndLabel(grCanvas *canvas)
{
    double boxwidth = 1.0/dataset->dims;
    double boxheight = 1.0/dataset->dims;

	//draw grid
    canvas->SetForeground(interface_color[GRID1]);
    canvas->SetXorMode(false);

    for (int i = 0;i < dataset->dims;i++)
        for (int j = 0;j < dataset->dims;j++)
            canvas->DrawRectangle(i*boxwidth, j*boxheight, boxwidth, boxheight);

	// draw labels
    canvas->SetForeground(interface_color[TEXT]);

    Vec2 font_size = canvas->GetDataFontSize("X");
    double max_str_size = boxwidth - font_size.X;

    // draw horizontal labels
    // add letters one at a time until space runs out
    for (i=0; i<dataset->dims; i++)
    {
        double x = i*boxwidth;
        double y = .8*font_size.Y;
        canvas->DrawConfinedString(x,y,dataset->names[i],max_str_size);
    }

    // draw vertical labels
    // add letters one at a time until space runs out
    for (i=0; i<dataset->dims; i++)
    {
        // initial coordinates
        Vec2 p;
        p.X = -1.5*font_size.X;
        p.Y = i*boxheight; // - font_size.Y*1.1;

        for (int j=0; j<int(strlen(dataset->names[i])); j++)
        {
            Vec2 letter = canvas->GetDataFontSize(&dataset->names[i][j],1);
            p.Y -= 1.2*letter.Y;

            if (p.Y > (i+1)*boxheight+letter.Y) break;

            canvas->DrawString(p.X,p.Y,&dataset->names[i][j],1);
        }
    }
}
*/

void CHierScatter::PrepareCanvas()
{
    //jing need to do 
    CDisplay::PrepareCanvas();
    if ( show_wide_axes )
        DrawScatterGridWideAxes(canvas);
    else
        DrawScatterGrid(canvas);
//    DrawScatterGrid(canvas);
    canvas->SetXorMode(false);
//    glPointSize((float) ((DSIZE+1)*canvas->zoom.X));
    glPointSize((float) ((DSIZE+1)));
}


void CHierScatter::InputMouseMotion(int x, int y)
{
    if (!dataset) return;

    if ( screen_distortion->distortion_status )
    {
        InputScatterFindPlot(canvas, x, y, dimx, dimy);
        return;
    }

    ScatterCaption(canvas, x, y);
    static int prev_sx, prev_sy;

    switch (input_mode) {
        case CANVAS_PAN:
            appCanvasMousePan(canvas);
            break;

        default: break;
    }
    prev_sx = x;
    prev_sy = y;
}


void CHierScatter::InputButtonPress(int button, int x, int y)
{
    if (!dataset) return;

    if ( screen_distortion->distortion_status )
    {
        return;
    }

    if (canvas->button[2]==true) {
        input_mode = CANVAS_PAN;
        canvas->SetHandCursor();
        return;
    }

    InputMouseMotion(x,y);
}

void CHierScatter::InputButtonRelease(int button, int x, int y)
{
    if (!dataset) return;

    if ( screen_distortion->distortion_status )
    {
        double delta;

        input_mode = INCREASE_PLOT_SIZE;
        switch (input_mode) 
        {
            case INCREASE_PLOT_SIZE:
                delta = 1. / dataset->dims * 0.2;
                InputScatterScalePlotSize(canvas, dimx, dimy, delta);
                canvas->Refresh();
                break;
            case DECREASE_PLOT_SIZE:
                delta = (-1.) / dataset->dims * 0.2;
                InputScatterScalePlotSize(canvas, dimx, dimy, delta);
                break;
            default: break;
        }
        return;
    }

    switch (input_mode) {
        case CANVAS_PAN:
            canvas->SetDefaultCursor();
            break;

        default: break;
    }

    input_mode = NONE;
    xor_needs_undo = false;
}


bool CHierScatter::TestDimensionality()  
{
	return ScatterTestDimensionality();
}