//modified by Jing Yang after 7/6/2001
#include "ogl.H"
#include "CSunburstDisplay.H"
#include "XmdvTool.H"
#include "colormap.H"
#include "magnify.H"
#include "brush_oper.H"
#include "CDimReducer.H"

#define RAD_DEGREE 0.0174532925199

CSunburstDisplay::CSunburstDisplay()
{
	near_mouse_mode = INACTIVE;
	mouse_mode = INACTIVE;

    tree = NULL;
    prev_node = NULL;
}


CSunburstDisplay::~CSunburstDisplay()
{
}


//draw the canvas;
void CSunburstDisplay::Draw()
{
    if ( dim_reduction_info->enable )
        tree= GetTree();
    if ( !tree )
        return;

	//set parameters used for drawing, to be rewritten in subclasses;
	SetParameters();
	//set canvas background, clear canvas, and draw axes, to be rewritten in subclasses;
	PrepareCanvas();

    glShadeModel(GL_SMOOTH);
	//draw clusters of sunburst;
	TraceDrawClusters(tree->root);
	//draw cluster outlines of sunburst;
	TraceDrawClusterOutlines(tree->root);
	//draw names for selected clusters
    TraceDrawClusterNames(tree->root);
	//draw circles of sunburst;
//	DrawCircles();
    //free space used for drawing, to be rewritten in subclasses;
	FreeSpace();
}

CSunburstClusterTree* CSunburstDisplay::GetTree()
{
    return NULL;
}


void CSunburstDisplay::TraceDrawClusters( CSunburstCluster *node )
{
	if ( node == NULL )
		return;
    DrawSingleCluster(node);
	CSunburstCluster *sibling = node->siblings;
	TraceDrawClusters(sibling);

    if ( node->show_children == true )
    {
	    CSunburstCluster *child = node->children;
	    TraceDrawClusters(child);
    }
}


void CSunburstDisplay::TraceDrawClusterOutlines( CSunburstCluster *node )
{
	if ( node == NULL )
		return;
    DrawSingleClusterOutline(node);
	CSunburstCluster *sibling = node->siblings;
	TraceDrawClusterOutlines(sibling);

    if ( node->show_children == true )
    {
	    CSunburstCluster *child = node->children;
	    TraceDrawClusterOutlines(child);
    }
}


void CSunburstDisplay::TraceDrawClusterNames( CSunburstCluster *node )
{
	if ( node == NULL )
		return;
    DrawSingleClusterName(node);
	CSunburstCluster *sibling = node->siblings;
	TraceDrawClusterNames(sibling);

    if ( node->show_children == true )
    {
	    CSunburstCluster *child = node->children;
	    TraceDrawClusterNames(child);
    }
}


Vec2 CSunburstDisplay::MapData(double radius, double angle)
{
	angle -= 90;
	angle *= RAD_DEGREE;
	Vec2 p;
	p.X = center.X + radius * cos(angle);
	p.Y = center.Y - radius * sin(angle);
	
	return p;
}

/*
//draw a single cluster
void CSunburstDisplay::DrawSingleCluster(CSunburstCluster *node)
{

//	RGBt color = dtree->CMAP[node->color_idx];
    if ( node->selected == true )
        canvas->SetForeground(interface_color[HIGHLIGHT]);
    else
        canvas->SetForeground(node->color);
//    canvas->SetForeground(dtree->CMAP[node->color_idx]);
	canvas->DrawPartialDisk( center.X, center.Y, radius[node->level-1], radius[node->level], 
					 node->start_angle, node->sweep_angle);
}
*/

//draw a single cluster
void CSunburstDisplay::DrawSingleCluster(CSunburstCluster *node)
{
    canvas->SetForeground(node->color);
	canvas->DrawPartialDisk( center.X, center.Y, radius[node->level-1], radius[node->level], 
					 node->start_angle, node->sweep_angle);
    if ( node->selected == true )
    {
        canvas->SetForeground(interface_color[HIGHLIGHT1]);
	    canvas->DrawPartialDisk( center.X, center.Y, radius[node->level-1]+(radius[node->level]-radius[node->level-1])/5*2,
            radius[node->level-1]+(radius[node->level]-radius[node->level-1])/5*3, node->start_angle, node->sweep_angle);
//	    canvas->DrawWideArc( center.X, center.Y, radius[node->level-1]+(radius[node->level]-radius[node->level-1])/3, 
//            node->start_angle, node->sweep_angle, (radius[node->level]-radius[node->level-1])/3);
    }
}


void CSunburstDisplay::DrawSingleClusterName(CSunburstCluster *node)
{
}


//draw a single cluster outline
void CSunburstDisplay::DrawSingleClusterOutline(CSunburstCluster *node)
{
    canvas->SetForeground(interface_color[TEXT]);
	if ( fabs(node->sweep_angle - 360) > 1e-5 )
	{
		Vec2 p1 = MapData(radius[node->level-1], node->start_angle);
		Vec2 p2 = MapData(radius[node->level], node->start_angle);
		canvas->DrawLine(p1.X, p1.Y, p2.X, p2.Y);

		p1 = MapData(radius[node->level-1], node->GetEndAngle());
		p2 = MapData(radius[node->level], node->GetEndAngle());
		canvas->DrawLine(p1.X, p1.Y, p2.X, p2.Y);
	}

	canvas->DrawArc( center.X, center.Y, radius[node->level-1], node->start_angle, node->sweep_angle);
	canvas->DrawArc( center.X, center.Y, radius[node->level], node->start_angle, node->sweep_angle);

}

//set parameters used for the drawing; to be rewritten by subclasses;
void CSunburstDisplay::SetParameters()
{
	center.X = 0.5;
	center.Y = 0.5;
	radius.SetLen(tree->max_level+1);
	double radius_delta = 0.5 / (tree->max_level+1);
	radius[0] = radius_delta;
	for ( int i = 0; i < tree->max_level; i++ )
	{
		radius[i+1] = radius[i]+radius_delta;
	}

	circle_offset = 0.008/tree->max_level;
}


//free space used for drawing; to be rewritten by subclasses;
void CSunburstDisplay::FreeSpace()
{
}


void CSunburstDisplay::DrawCircles()
{
     canvas->SetForeground(interface_color[TEXT]);
	 for ( int i = 0; i <= tree->max_level; i++ )
	 {
		canvas->DrawCircle( center.X, center.Y, radius[i], circle_offset);
	 }
}

void CSunburstDisplay::GetLevelAndAngleAccordingXY(double x, double y, int &level, double &angle)
{
    Vec2 p = canvas->MapScreenToData( Vec2(x,y) );
	double distance = (p - center).norm();	
	
	if ( distance > 0.5 )
	{
		level = -1;
		return;
	}

	level = tree->max_level;
	while ( level > 0 && distance < radius[level-1] )
	{
		level--;
	}

	if ( level <= 0 )
		return;

	angle = acos((p.X - center.X)/distance);
	if ( ( center.Y - p.Y ) < 0 )
		angle *= -1;

	angle /= RAD_DEGREE;
	angle += 90;
	
	return;
}


bool CSunburstDisplay::GetAngleAccordingXY(double x, double y, double &angle)
{
    Vec2 p = canvas->MapScreenToData( Vec2(x,y) );
	double distance = (p - center).norm();	
	
	if ( distance < 1e-5 )
		return false;

	angle = acos((p.X - center.X)/distance);
	if ( ( center.Y - p.Y ) < 0 )
		angle *= -1;

	angle /= RAD_DEGREE;
	angle += 90;
	
	return true;;

}
//find if (x, y) is in a node;
CSunburstCluster* CSunburstDisplay::FindNode(double x, double y)
{
	int level;
	double angle;
	
	GetLevelAndAngleAccordingXY(x, y, level, angle);

	if ( level <= 0 )
		return NULL;

	return tree->FindNodeAccordingLevelAndAngle(level, angle);
}

//find if (x, y) is near the end of a node and in the node; 
CSunburstCluster* CSunburstDisplay::FindNode(double x, double y, bool &start, bool &end, bool &in)
{
	int level;
	double angle;

	GetLevelAndAngleAccordingXY(x, y, level, angle);

	if ( level <= 0 )
		return NULL;

	return tree->FindNodeAccordingLevelAndAngle(level, angle, start, end, in);
}

void CSunburstDisplay::SunburstCaption(int x, int y)
{
}

void CSunburstDisplay::InputMouseMotion(int x, int y)
{
}


void CSunburstDisplay::InputButtonPress(int button, int x, int y)
{
}

void CSunburstDisplay::InputButtonRelease(int button, int x, int y)
{
}

